@@ -37,15 +37,17 @@ get their definitions straight:
 """
 
 
-import logging
-import collections
-import itertools as it
+import                    logging
+import                    collections
+import collections.abc as abc
+import itertools       as it
 
 import numpy     as np
 import nibabel   as nib
 
-import fsl.utils.notifier as notifier
-import fsl.utils.async    as async
+import fsl.utils.notifier    as notifier
+import fsl.utils.naninfrange as nir
+import fsl.utils.idle        as idle
 
 
 log = logging.getLogger(__name__)
@@ -74,8 +76,7 @@ class ImageWrapper(notifier.Notifier):
      - The image data is not modified (via :meth:`__setitem__`.
 
     If any of these conditions do not hold, the image data will be loaded into
-    memory and accessed directly, via the ``nibabel.Nifti1Image.get_data``
-    method.
+    memory and accessed directly.
 
 
     *Image dimensionality*
@@ -137,7 +138,6 @@ class ImageWrapper(notifier.Notifier):
     .. autosummary::
        :nosignatures:
 
-       naninfrange
        isValidFancySliceObj
        canonicalSliceObj
        sliceObjToSliceTuple
@@ -175,6 +175,8 @@ class ImageWrapper(notifier.Notifier):
                         data range is updated directly on reads/writes.
         """
 
+        import fsl.data.image as fslimage
+
         self.__image      = image
         self.__name       = name
         self.__taskThread = None
@@ -187,10 +189,10 @@ class ImageWrapper(notifier.Notifier):
             if d == 1: self.__numRealDims -= 1
             else:      break
 
-        # Degenerate case - if every
-        # dimension has length 1
-        if self.__numRealDims == 0:
-            self.__numRealDims = len(image.shape)
+        # Degenerate case - less
+        # than three real dimensions
+        if self.__numRealDims < 3:
+            self.__numRealDims = min(3, len(image.shape))
 
         # And save the number of
         # 'padding' dimensions too.
@@ -203,7 +205,7 @@ class ImageWrapper(notifier.Notifier):
         # __getitem__ to force the
         # result to have the correct
         # dimensionality.
-        self.__canonicalShape = canonicalShape(image.shape)
+        self.__canonicalShape = fslimage.canonicalShape(image.shape)
 
         # The internal state is stored
         # in these attributes - they're
@@ -215,11 +217,15 @@ class ImageWrapper(notifier.Notifier):
 
         self.reset(dataRange)
 
-        if loadData:
+        # We keep an internal ref to
+        # the data numpy array if/when
+        # it is loaded in memory
+        self.__data = None
+        if loadData or image.in_memory:
             self.loadData()
 
         if threaded:
-            self.__taskThread = async.TaskThread()
+            self.__taskThread = idle.TaskThread()
             self.__taskThread.daemon = True
             self.__taskThread.start()
 
@@ -229,9 +235,10 @@ class ImageWrapper(notifier.Notifier):
         the :class:`.TaskThread` is stopped.
         """
         self.__image = None
+        self.__data  = None
         if self.__taskThread is not None:
             self.__taskThread.stop()
-            self.__taskThraed = None
+            self.__taskThread = None
 
 
     def getTaskThread(self):
@@ -298,7 +305,16 @@ class ImageWrapper(notifier.Notifier):
 
         # Internally, we calculate and store the
         # data range for each volume/slice/vector
-        self.__volRanges = np.zeros((nvols, 2), dtype=np.float32)
+        #
+        # We use nan as a placeholder, so the
+        # dtype must be non-integral. The
+        # len(dtype) check takes into account
+        # structured data (e.g. RGB)
+        dtype = self.__image.get_data_dtype()
+        if np.issubdtype(dtype, np.integer) or len(dtype) > 0:
+            dtype = np.float32
+        self.__volRanges = np.zeros((nvols, 2),
+                                    dtype=dtype)
 
         self.__coverage[ :] = np.nan
         self.__volRanges[:] = np.nan
@@ -365,16 +381,11 @@ class ImageWrapper(notifier.Notifier):
         """Forces all of the image data to be loaded into memory.
 
         .. note:: This method will be called by :meth:`__init__` if its
-                  ``loadData`` parameter is ``True``.
+                  ``loadData`` parameter is ``True``. It will also be called
+                  on all write operations (see :meth:`__setitem__`).
         """
-
-        # If the data is not already loaded, this will
-        # cause nibabel to load it. By default, nibabel
-        # will cache the numpy array that contains the
-        # image data, so subsequent calls to this
-        # method will not overwrite any changes that
-        # have been made to the data array.
-        self.__image.get_data()
+        if self.__data is None:
+            self.__data = np.asanyarray(self.__image.dataobj)
 
 
     def __getData(self, sliceobj, isTuple=False):
@@ -395,14 +406,7 @@ class ImageWrapper(notifier.Notifier):
         # ArrayProxy. Otheriwse if it is in
         # memory, we can access it directly.
         #
-        # Furthermore, if it is in memory and
-        # has been modified, the ArrayProxy
-        # will give us out-of-date values (as
-        # the ArrayProxy reads from disk). So
-        # we have to read from the in-memory
-        # array to get changed values.
-        #
-        # Finally, note that if the caller has
+        # Note also that if the caller has
         # given us a 'fancy' slice object (a
         # boolean numpy array), but the image
         # data is not in memory, we can't access
@@ -410,8 +414,8 @@ class ImageWrapper(notifier.Notifier):
         # (the dataobj attribute) cannot handle
         # fancy indexing. In this case an error
         # will be raised.
-        if self.__image.in_memory: return self.__image.get_data()[sliceobj]
-        else:                      return self.__image.dataobj[   sliceobj]
+        if self.__data is not None: return self.__data[         sliceobj]
+        else:                       return self.__image.dataobj[sliceobj]
 
 
     def __imageIsCovered(self):
@@ -432,18 +436,18 @@ class ImageWrapper(notifier.Notifier):
         _, expansions = calcExpansion(slices, self.__coverage)
         expansions    = collapseExpansions(expansions, self.__numRealDims - 1)
 
-        log.debug('Updating image {} data range [slice: {}] '
-                  '(current range: [{}, {}]; '
-                  'number of expansions: {}; '
-                  'current coverage: {}; '
-                  'volume ranges: {})'.format(
-                      self.__name,
-                      slices,
-                      self.__range[0],
-                      self.__range[1],
-                      len(expansions),
-                      self.__coverage,
-                      self.__volRanges))
+        log.debug('Updating image %s data range [slice: %s] '
+                  '(current range: [%s, %s]; '
+                  'number of expansions: %s; '
+                  'current coverage: %s; '
+                  'volume ranges: %s)',
+                  self.__name,
+                  slices,
+                  self.__range[0],
+                  self.__range[1],
+                  len(expansions),
+                  self.__coverage,
+                  self.__volRanges)
 
         # As we access the data for each expansions,
         # we want it to have the same dimensionality
@@ -459,22 +463,24 @@ class ImageWrapper(notifier.Notifier):
         # the min/max per volume/expansion, and
         # iteratively update the stored per-volume
         # coverage and data range.
-        for i, exp in enumerate(expansions):
-
-            data = self.__getData(exp, isTuple=True)
-            data = data.squeeze(squeezeDims)
+        for exp in expansions:
 
+            data     = self.__getData(exp, isTuple=True)
+            data     = data.squeeze(squeezeDims)
             vlo, vhi = exp[self.__numRealDims - 1]
 
             for vi, vol in enumerate(range(vlo, vhi)):
 
                 oldvlo, oldvhi = self.__volRanges[vol, :]
                 voldata        = data[..., vi]
+                newvlo, newvhi = nir.naninfrange(voldata)
 
-                newvlo, newvhi = naninfrange(voldata)
-
-                if (not np.isnan(oldvlo)) and oldvlo < newvlo: newvlo = oldvlo
-                if (not np.isnan(oldvhi)) and oldvhi > newvhi: newvhi = oldvhi
+                if np.isnan(newvlo) or \
+                   (not np.isnan(oldvlo) and oldvlo < newvlo):
+                    newvlo = oldvlo
+                if np.isnan(newvhi) or \
+                   (not np.isnan(oldvhi) and oldvhi > newvhi):
+                    newvhi = oldvhi
 
                 # Update the stored range and
                 # coverage for each volume
@@ -485,7 +491,7 @@ class ImageWrapper(notifier.Notifier):
         # Calculate the new known data
         # range over the entire image
         # (i.e. over all volumes).
-        newmin, newmax = naninfrange(self.__volRanges)
+        newmin, newmax = nir.naninfrange(self.__volRanges)
 
         oldmin, oldmax = self.__range
         self.__range   = (newmin, newmax)
@@ -493,12 +499,12 @@ class ImageWrapper(notifier.Notifier):
 
         if any((oldmin is None, oldmax is None)) or \
            not np.all(np.isclose([oldmin, oldmax], [newmin, newmax])):
-            log.debug('Image {} range changed: [{}, {}] -> [{}, {}]'.format(
-                self.__name,
-                oldmin,
-                oldmax,
-                newmin,
-                newmax))
+            log.debug('Image %s range changed: [%s, %s] -> [%s, %s]',
+                      self.__name,
+                      oldmin,
+                      oldmax,
+                      newmin,
+                      newmax)
             self.notify()
 
 
@@ -577,15 +583,15 @@ class ImageWrapper(notifier.Notifier):
             slices = np.array(slices.T, dtype=np.uint32)
             slices = tuple(it.chain(map(tuple, slices), [(lowVol, highVol)]))
 
-            log.debug('Image {} data written - clearing known data '
-                      'range on volumes {} - {} (write slice: {}; '
-                      'coverage: {}; volRanges: {})'.format(
-                          self.__name,
-                          lowVol,
-                          highVol,
-                          slices,
-                          self.__coverage[:, :, lowVol:highVol],
-                          self.__volRanges[lowVol:highVol, :]))
+            log.debug('Image %s data written - clearing known data '
+                      'range on volumes %s - %s (write slice: %s; '
+                      'coverage: %s; volRanges: %s)',
+                      self.__name,
+                      lowVol,
+                      highVol,
+                      slices,
+                      self.__coverage[:, :, lowVol:highVol],
+                      self.__volRanges[lowVol:highVol, :])
 
             for vol in range(lowVol, highVol):
                 self.__coverage[:, :, vol]    = np.nan
@@ -608,7 +614,7 @@ class ImageWrapper(notifier.Notifier):
         :arg sliceobj: Something which can slice the image data.
         """
 
-        log.debug('Getting image data: {}'.format(sliceobj))
+        log.debug('Getting image data: %s', sliceobj)
 
         shape              = self.__canonicalShape
         realShape          = self.__image.shape
@@ -685,13 +691,13 @@ class ImageWrapper(notifier.Notifier):
 
             # If we are slicing a scalar, the
             # assigned value has to be scalar.
-            if expNdims == 0 and isinstance(values, collections.Sequence):
+            if expNdims == 0 and isinstance(values, abc.Sequence):
 
                 if len(values) > 1:
                     raise IndexError('Invalid assignment: [{}] = {}'.format(
                         sliceobj, len(values)))
 
-                values = values[0]
+                values = np.array(values).flatten()[0]
 
             # Make sure that the values
             # have a compatible shape.
@@ -707,53 +713,10 @@ class ImageWrapper(notifier.Notifier):
         # have any effect.
         self.loadData()
 
-        self.__image.get_data()[sliceobj] = values
+        self.__data[sliceobj] = values
         self.__updateDataRangeOnWrite(slices, values)
 
 
-def naninfrange(data):
-    """Returns the minimum and maximum values in the given ``numpy`` array,
-    ignoring ``nan`` and ``inf`` values.
-
-    The ``numpy.nanmin``/``numpy.nanmax`` functions do not handle
-    positive/negative infinity, so if such values are in the data, we need to
-    use an alternate approach to calculating the minimum/maximum.
-    """
-
-    if not np.issubdtype(data.dtype, np.float):
-        return data.min(), data.max()
-
-    # But np.nanmin/nanmax are substantially
-    # faster than the alternate, so we try it
-    # first.
-    dmin = np.nanmin(data)
-    dmax = np.nanmax(data)
-
-    # If there are no nans/infs in the data,
-    # we can just use nanmin/nanmax
-    if np.isfinite(dmin) and np.isfinite(dmax):
-        return dmin, dmax
-
-    # The entire array contains nans
-    if np.isnan(dmin):
-        return dmin, dmin
-
-    # Otherwise we need to calculate min/max
-    # only on finite values. This is the slow
-    # option.
-
-    # Find all finite values
-    finite = np.isfinite(data)
-
-    # Try to calculate min/max on those values.
-    # An error will be raised if there are no
-    # finite values in the array
-    try:
-        return data[finite].min(), data[finite].max()
-    except:
-        return np.nan, np.nan
-
-
 def isValidFancySliceObj(sliceobj, shape):
     """Returns ``True`` if the given ``sliceobj`` is a valid and fancy slice
     object.
@@ -795,29 +758,6 @@ def canonicalSliceObj(sliceobj, shape):
         return nib.fileslice.canonical_slicers(sliceobj, shape)
 
 
-def canonicalShape(shape):
-    """Calculates a *canonical* shape, how the given ``shape`` should
-    be presented. The shape is forced to be at least three dimensions,
-    with any other trailing dimensions of length 1 ignored.
-    """
-
-    shape = list(shape)
-
-    # Squeeze out empty dimensions, as
-    # 3D image can sometimes be listed
-    # as having 4 or more dimensions
-    for i in reversed(range(len(shape))):
-        if shape[i] == 1: shape = shape[:i]
-        else:             break
-
-    # But make sure the shape
-    # has at 3 least dimensions
-    if len(shape) < 3:
-        shape = shape + [1] * (3 - len(shape))
-
-    return shape
-
-
 def expectedShape(sliceobj, shape):
     """Given a slice object, and the shape of an array to which
     that slice object is going to be applied, returns the expected
@@ -866,6 +806,8 @@ def expectedShape(sliceobj, shape):
         if start is None: start = 0
         if stop  is None: stop  = shape[i]
 
+        stop = min(stop, shape[i])
+
         expShape.append(stop - start)
 
     return len(expShape), expShape
@@ -888,7 +830,7 @@ def sliceObjToSliceTuple(sliceobj, shape):
 
     # The sliceobj could be a single sliceobj
     # or integer, instead of a tuple
-    if not isinstance(sliceobj, collections.Sequence):
+    if not isinstance(sliceobj, abc.Sequence):
         sliceobj = [sliceobj]
 
     # Turn e.g. array[6] into array[6, :, :]
@@ -1097,7 +1039,7 @@ def calcExpansion(slices, coverage):
     # 'padding' dimensions of size 1.
     def finishExpansion(exp, vol):
         exp.append((vol, vol + 1))
-        for i in range(padDims):
+        for _ in range(padDims):
             exp.append((0, 1))
         return exp
 
