@@ -4,7 +4,7 @@
 #
 # Author: Paul McCarthy <pauldmccarthy@gmail.com>
 #
-"""
+"""This module contains functions for moving/copying NIFIT image files.
 
 .. autosummary::
    :nosignatures:
@@ -18,6 +18,8 @@ import                   os
 import os.path        as op
 import                   shutil
 
+import nibabel        as nib
+
 import fsl.utils.path as fslpath
 import fsl.data.image as fslimage
 
@@ -53,11 +55,16 @@ def imcp(src,
                         copied. See :func:`immv`.
     """
 
-    import nibabel as nib
+    # special case - non-existent directory
+    if dest.endswith('/') and not op.isdir(dest):
+        raise fslpath.PathError('Directory does not exist: {}'.format(dest))
 
     if op.isdir(dest):
         dest = op.join(dest, op.basename(src))
 
+    src  = op.abspath(src)
+    dest = op.abspath(dest)
+
     srcBase,  srcExt  = fslimage.splitExt(src)
     destBase, destExt = fslimage.splitExt(dest)
 
@@ -130,9 +137,16 @@ def imcp(src,
 
         img = nib.load(src)
         nib.save(img, dest)
+        img = None
 
         if move:
-            os.remove(src)
+            # if input is an image pair, we
+            # need to remove all input files
+            srcs = fslpath.getFileGroup(src,
+                                        fslimage.ALLOWED_EXTENSIONS,
+                                        fslimage.FILE_GROUPS)
+            for src in srcs:
+                os.remove(src)
 
         return
 
