@@ -81,8 +81,8 @@ class WeakFunctionRef(object):
         irreconcilably incompatible (or just where :mod:`six` does not have a
         function to help us).
 
-        In Python 2 there is no difference between an unbound method and a
-        function. But in Python 3, an unbound method is still a method (and
+        In Python 3 there is no difference between an unbound method and a
+        function. But in Python 2, an unbound method is still a method (and
         inspect.ismethod returns True).
         """
 
@@ -132,6 +132,11 @@ class WeakFunctionRef(object):
         return None
 
 
+    def __call__(self):
+        """See :meth:``function``. """
+        return self.function()
+
+
     def function(self):
         """Return a reference to the encapsulated function or method,
         or ``None`` if the function has been garbage collected.
@@ -148,9 +153,9 @@ class WeakFunctionRef(object):
         obj = self.obj()
 
         # Return the bound method object
-        try:    return getattr(obj, self.funcName)
+        try: return getattr(obj, self.funcName)
 
         # If the function is a bound private method,
         # its name on the instance will have been
         # mangled, so we need to search for it
-        except: return self.__findPrivateMethod()
+        except AttributeError: return self.__findPrivateMethod()
